const { DISPATCHER: { SUBSCRIPTIONS } } = require('enums').OBJECT_TYPE;

const { eventsConsumer } = require('../../providers/eventsConsumer');
const { EventSubscription } = require('../../models/EventSubscription');

const Utils = require('./utils');
const Api = require('./api');

class AddonState {
    /**
     *
     * @param {function} notificator
     * @return {Promise<void>}
     */
    constructor(notificator) {
        /**
         * Add-on state
         * @type {{trackables: Object, subscribers: Object,  notifications: Object,  sublocations: Object}}
         * @example {
            'trackables': {
                '1-w20IkMRumeVyd5UnpRKA': {
                    'name': 'tko_1a',       'type': 'NAV', 'haveNode': true,  'responsive': false,
                    'category': 'UNSPECIFIED', sublocationUid: 'Ddfgfs45dg234hsfhDFADF', projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                 },
                'W-757HuZQBuS516tjvdVog': {
                    'name': 'persone sss',  'type': 'NAV', 'haveNode': false, 'responsive': false,
                    'category': 'PERSON'     , sublocationUid: 'Ddfgfs45dg234hsfhDFADF', projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                 },
                'Z_YaKVMaQ6yNJyNUwJ3jnw': {
                    'name': 'new trko',     'type': 'NAV', 'haveNode': true,  'responsive': false,
                    'category': 'PERSON'     , sublocationUid: 'Ddfgfs45dg234hsfhDFADF', projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                }
            },
            'subscribers': {
                'yXnnBpcXTYuy0Taex-GpQQ': {
                    'name': 'ssdf',
                    'addresses': { phone: '065666666666', email: 'lab+1@redpointpositioning.com' },
                    'notifications': ['gdDv4_BFSAKeWTnRbr2SbA'],
                     projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                }
            },
            'notifications': {
                'gdDv4_BFSAKeWTnRbr2SbA': {
                    'name': 'Facility_out_notifications',
                    'conditions': { 'zone_severity_level': 1, 'alarm_type_list': ['trackable_unresponsive'] },
                    'active': 1,
                    'types': { 'email': true, 'phone': true },
                    'notificationType': 'ALARM',
                     projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                }
            }
            'sublocations': {
                'Ddfgfs45dg234hsfhDFADF': {
                    'name': 'sublocaiton name',
                     projectUid: 'gPrv4_BFSAKeWTnRbr2SbA'
                }
            }
         }
         */
        this.state = {
            trackables: null,
            subscribers: null,
            notifications: null,
            sublocations: null,
        };
        this.notify = notificator;
    }
    
    async init() {
        await this.loadEntitiesFromApi();
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.TRACKABLE_CREATED, this.trackableCreateHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.TRACKABLE_UPDATED, this.trackableUpdateHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.TRACKABLE_DELETED, this.trackableDeleteHandler.bind(this)));
    
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.NOTIFICATION_CREATED, this.notificationCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.NOTIFICATION_UPDATED, this.notificationCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.NOTIFICATION_DELETED, this.notificationDeletedHandler.bind(this)));
    
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBSCRIBER_CREATED, this.subscriberCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBSCRIBER_UPDATED, this.subscriberCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBSCRIBER_DELETED, this.subscriberDeletedHandler.bind(this)));
        
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBLOCATION_CREATED, this.sublocationCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBLOCATION_UPDATED, this.sublocationCreatedUpdatedHandler.bind(this)));
        eventsConsumer.subscribe(new EventSubscription(SUBSCRIPTIONS.SUBLOCATION_DELETED, this.sublocationDeletedHandler.bind(this)));
    }
    
    async loadEntitiesFromApi() {
        if (!this.state.trackables) {
            const trackables = await Api.loadTrackables();
            this.state.trackables = (trackables || []).reduce((memo, tko) => {
                memo[tko.uid] = Utils.parseTrackable(tko);
                return memo;
            }, {});
        }
        if (!this.state.subscribers) {
            const subscribers = await Api.loadSubscribers();
            this.state.subscribers = (subscribers || []).reduce((memo, subscriber) => {
                memo[subscriber.uid] = Utils.parseSubscriber(subscriber);
                return memo;
            }, {});
        }
        if (!this.state.notifications) {
            const notifications = await Api.loadNotifications();
            this.state.notifications = (notifications || []).reduce((memo, notification) => {
                memo[notification.uid] = Utils.parseNotification(notification);
                return memo;
            }, {});
        }
        if (!this.state.sublocations) {
            const sublocations = await Api.loadSublocations();
            this.state.sublocations = (sublocations || []).reduce((memo, sublocation) => {
                memo[sublocation.uid] = Utils.parseSublocation(sublocation);
                return memo;
            }, {});
        }
    }
    
    testSetState({ trackables, subscribers, notifications, sublocations }) {
        this.state = {
            trackables,
            subscribers,
            notifications,
            sublocations,
        };
        return this.state;
    }
    
    trackableCreateHandler(notificationObject) {
        const tko = notificationObject.plist;
        this.state.trackables[tko.uid] = Utils.parseTrackable(tko);
    }
    trackableUpdateHandler(notificationObject) {
        const tko = notificationObject.plist;
        const updatedTko = Utils.parseTrackable(tko);
        const oldTko = { ...this.state.trackables[tko.uid] };
        this.state.trackables[tko.uid] = updatedTko;
    
        if (Utils.isTkoFacilityOut(oldTko, updatedTko)) {
            const emailRecipients = Utils.getSendingDestination(this.state.subscribers, this.state.notifications, updatedTko.projectUid);
            const sublocation = this.state.sublocations[updatedTko.sublocationUid] || {};

            this.notify(emailRecipients, tko.name, sublocation.name);
        }
    }
    trackableDeleteHandler(notificationObject) {
        const tko = notificationObject.plist;
        delete this.state.trackables[tko.uid];
    }
    
    notificationCreatedUpdatedHandler(notificationObject) {
        const notification = notificationObject.plist;
        this.state.notifications[notification.uid] = Utils.parseNotification(notification);
    }
    notificationDeletedHandler(notificationObject) {
        const notification = notificationObject.plist;
        delete this.state.notifications[notification.uid];
    }
    
    subscriberCreatedUpdatedHandler(notificationObject) {
        const subscriber = notificationObject.plist;
        this.state.subscribers[subscriber.uid] = Utils.parseSubscriber(subscriber);
    }
    subscriberDeletedHandler(notificationObject) {
        const subscriber = notificationObject.plist;
        delete this.state.subscribers[subscriber.uid];
    }
    
    sublocationCreatedUpdatedHandler(notificationObject) {
        const sublocation = notificationObject.plist;
        this.state.sublocations[sublocation.uid] = Utils.parseSubscriber(sublocation);
    }
    sublocationDeletedHandler(notificationObject) {
        const sublocation = notificationObject.plist;
        delete this.state.sublocations[sublocation.uid];
    }
}

module.exports = {
    AddonState,
};
