const Get = require('lodash.get');

const Api = require('./api');
const constants = require('./constants');

function isTkoResponsive(tko) {
    if (!tko.node) {
        return true;
    }
    return Get(tko, 'node.responsive', false);
}

/**
 *
 * @param tko
 * @property {String|Number} tko.ts
 * @property {Undefined|Number} tko.ms
 * @return {number}
 */
function getPositionTimestamp(tko) {
    if (tko.position && tko.position.ts && typeof tko.position.ts === 'number' && typeof tko.position.ms === 'number') {
        return (Get(tko, 'position.ts', 0) * 1000) + Get(tko, 'position.ms', 0);
    } else if (tko.position && tko.position.ts && typeof tko.position.ts === 'string') {
        return new Date(tko.position.ts).getTime();
    }
    return 0;
}

function getSublocation(tko) {
    const positionTimestamp = getPositionTimestamp(tko);
    const heartbeatTimestamp = (Get(tko, 'heartbeat.ts', 0) * 1000) + Get(tko, 'heartbeat.ms', 0);
    return positionTimestamp > heartbeatTimestamp ? Get(tko, 'position.sl_id', '') : Get(tko, 'heartbeat.sl_id', '');
}

function parseTrackable(tko) {
    return {
        name: tko.name,
        type: tko.type,
        haveNode: !!tko.node,
        responsive: isTkoResponsive(tko),
        category: tko.category,
        sublocationUid: getSublocation(tko),
        projectUid: tko.project_uid,
    };
}

function parseSubscriber(subscriber) {
    return {
        name: subscriber.name,
        addresses: subscriber.addresses,
        notifications: subscriber.notifications,
        projectUid: subscriber.project_uid,
    };
}

function parseNotification(notification) {
    return {
        name: notification.name,
        conditions: notification.conditions,
        active: notification.active,
        types: notification.types,
        notificationType: notification.notification_type,
        projectUid: notification.project_uid,
    };
}

function parseSublocation(sublocation) {
    return {
        name: sublocation.name,
        projectUid: sublocation.project_uid,
    };
}

/**
 * @description should send notification if category changed from asset/vehicle to person/unspecified
 *   and responsive changed from false to true
 * @param {Object} oldTko
 * @param {String} oldTko.name
 * @param {String} oldTko.type
 * @param {Boolean} oldTko.haveNode
 * @param {Boolean} oldTko.responsive
 * @param {String} oldTko.category
 * @param {Object} updatedTko
 * @param {String} updatedTko.name
 * @param {String} updatedTko.type
 * @param {Boolean} updatedTko.haveNode
 * @param {Boolean} updatedTko.responsive
 * @param {String} updatedTko.category
 * @returns {boolean}
 */
function isTkoFacilityOut(oldTko, updatedTko) {
    const isAssignChanged = oldTko.haveNode === updatedTko.haveNode;
    const isSameCategory = updatedTko.category === oldTko.category;
    const correctCategory = updatedTko.category === constants.CATEGORY.PERSON || updatedTko.category === constants.CATEGORY.UNSPECIFIED;
    const changesFromResponsiveToUnresponsive = oldTko.responsive && updatedTko.responsive === false;
    
    return isAssignChanged && correctCategory && isSameCategory && changesFromResponsiveToUnresponsive;
}

function getFacilityOutNotifications(notifications) {
    return Object.keys(notifications)
        .reduce((memo, key) => {
            const contains = Get(notifications, `[${key}].conditions.alarm_type_list`, '').includes(constants.TRACKABLE_UNRESPONSIVE);
            if (contains && notifications[key].active) {
                memo.push({
                    types: notifications[key].types,
                    projectUid: notifications[key].projectUid,
                    uid: key,
                });
            }
            return memo;
        }, []);
}

/**
 *
 * @param {Object} subscribers
 * @param {String} subscribers.name
 * @param {Object} subscribers.addresses
 * @param {Array<String{22}>} subscribers.notifications
 * @param {Object} notifications
 * @param {String} notifications.name
 * @param {Object} notifications.conditions
 * @param {Array<String{22}>} notifications.conditions.alarm_type_list
 * @param {Object} notifications.active
 * @param {Object} notifications.types
 * @param {Boolean} notifications.types.email
 * @param {Boolean} notifications.types.phone
 * @param {String} notifications.notificationType
 * @param {String} projectUid
 * @returns {Object<{emails: Array<String>, phones: Array<String>}>}
 */
function getSendingDestination(subscribers, notifications, projectUid) {
    const activeNotifications = getFacilityOutNotifications(notifications);
    return Object.keys(subscribers)
        .reduce((memo, key) => {
            // get notifications that is subscribed by subscribers
            const contains = activeNotifications.filter(x => (subscribers[key].notifications.some(y => x.uid === y)));

            contains.forEach((contains_item) => {
                const sameProject = subscribers[key].projectUid === projectUid;
                if (sameProject) {
                    const phone = contains_item.types.phone && subscribers[key].addresses.phone;
                    if (phone) { memo.phones[phone] = phone; }
    
                    const email = contains_item.types.email && subscribers[key].addresses.email;
                    if (email) { memo.emails[email] = email; }
                }
            });
            
            return memo;
        }, { emails: {}, phones: {} });
}

/**
 * Sending emails and/or sms to recipients
 * @param {Object<{emails: Array<String>, phones: Array<String>}>} recipients
 * @param {String} trackable_name
 * @param {String} sublocation_name
 */
async function sendRecipients(recipients, trackable_name, sublocation_name) {
    const { emails, phones } = recipients;
    const message = `Trackable ${trackable_name} became unresponsive in ${sublocation_name} area`;
    
    Object.keys(emails).forEach((email) => {
        Api.apiCreateEmail(message, email, 'Addon attendance detection email');
    });
    Object.keys(phones).forEach((phone) => {
        Api.apiCreateSms(message, phone);
    });
}

module.exports.isTkoResponsive = isTkoResponsive;
module.exports.parseTrackable = parseTrackable;
module.exports.parseSubscriber = parseSubscriber;
module.exports.parseNotification = parseNotification;
module.exports.parseSublocation = parseSublocation;
module.exports.isTkoFacilityOut = isTkoFacilityOut;
module.exports.sendRecipients = sendRecipients;
module.exports.getSendingDestination = getSendingDestination;
module.exports.getFacilityOutNotifications = getFacilityOutNotifications;
